/**
    Battery analyzer
    (c) 2016-2017 F. Štefanec
    This project is provided WITHOUT ANY WARRANTY, use at your own risk.
**/


#ifndef MAIN_H_INCLUDED
#define MAIN_H_INCLUDED

/** Library inclusions **/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>
#define uint_farptr_t unsigned long int // to fix some undefined stuff
#include <avr/pgmspace.h>

#define DEFAULT_SETTINGS 0

uint8_t x, y, z; /// general variables


/** Status and menu variables **/

uint8_t ad_preavg_count;
uint8_t stat, stat2, settings; // status variable
uint8_t submenu_stat = 0;
uint8_t field = 0;
uint8_t menu_pos = 1, old_menu_pos = 1, menu_pos2, old_menu_pos2;
volatile uint8_t beep_counter, old_beep_counter;
uint8_t rin_current_choice = 0;
const uint8_t rin_current1[10] PROGMEM = {0,  25,   0,  50,   0, 100, 150,   0,  50,   0};
const uint8_t rin_current2[10] PROGMEM = {25, 50,  50, 100, 100, 200, 250, 200, 250, 250};
uint8_t led_state1 = 2;
uint8_t led_state2 = 0;
uint16_t led_blink_time = 0;
uint8_t last_option = 0;
uint8_t fan_control = 0;

/** Timing related variables **/

volatile uint8_t seconds;
volatile uint8_t minutes;
volatile uint8_t hours;
unsigned long long elapsed_secs = 0; // max 917999 seconds
uint8_t tim0_sw_pre; // software prescaler for Timer 0
uint8_t tim0_even = 0;
uint16_t tim0_blink_counter = 0;


/** Button handling variables **/

uint8_t buttons_old_pin;
uint8_t buttons;
uint8_t buttons_old;
uint8_t buttons_pressed;
uint8_t button_repeat_counter;
uint8_t btn_debounce = 0; // button debounce counter

/** Display variables **/

uint8_t batt_icon = 0; // discharging battery icon
char buf[20]; // LCD buffer
char disch_power_str[8];
uint8_t display_mode; // display mode: current and mAh or power and mWh

/** Measurement variables **/

uint16_t end_voltage = 80; // end voltage in mV
uint16_t term_voltage = 80; // termination voltage in 10mV
volatile uint16_t adc_voltage = 0; // voltage in 10mV, max 20.46V (>20V = error)
uint16_t disp_voltage = 0;
volatile uint16_t supp_voltage = 0; // AD value for supply voltage
volatile uint16_t temperature_value = 0; // AD value for temperature
uint16_t volt_buf[64];
uint8_t ad_buf_ptr = 0;
uint16_t ad_avg_buf[32];
uint8_t ad_avg_buf_ptr;
uint8_t ad_throwaway;
uint8_t pwm_current = 1; // current in 10mA, max 2.55A
uint16_t v1, v2, i1, i2=25, rin; // (Rin measurement)


unsigned long long mAs = 0; // milliampseconds (max 917999*2550 = 2340897450 mAs)
unsigned long long mAh100 = 0; // milliamphours*100 (max 2340897450/36 = 65024929 mAh100 - to give 0.01mAh accuracy)
unsigned long long mWsx10 = 0; // divide by 36000 to get mWh

int8_t offset_comp; // offset compensation, in 0,000625 volts, this will be added to the displayed voltage

// MAX

// 2.55A 20.46V 255 2046


/** Function prototypes **/

void terminate_discharge();
void print_discharge_result();
void calculate_mAh();
void calculate_mWsx10();
void mAh_string(char *buf);
void mWh_string(char *buf);
void time_string(char *buf);
void voltage_string(char *buf, uint16_t voltage);
void current_string(char *buf, uint8_t current);
void watt_string(char* buf, uint16_t voltage, uint8_t current);
void display_prog();
void display_ccload();
void terminate_discharge();
void enable_1s_timer();
void fast_1s_timer();
void disable_1s_timer();
void set_current(uint8_t current);
void rin_test();

void lcd_print_pmem(const char *data, uint8_t length);
void lcd_print_pmem_setpos(const char *data, uint8_t len, uint8_t column, uint8_t line);
uint8_t get_buttons();
void clear_time();
void dly_1sec();

void enable_wdt();
void disable_wdt();

void deconf_regs();
void halt_error();

void cap_submenu();
void cc_submenu();
void rin_submenu();
void settings_submenu();
void escape_to_menu();
void print_menu(const char **items, uint8_t position);
void service_menu();
void disp_menu();
void print_settings_submenu(uint8_t cursor_pos);
void calib_menu();

void save_eeprom();
void set_offset_comp();
void save_eeprom();
void load_eeprom();

void print_temp_error();

#endif
